# Copyright (c) 2017 Intel, Inc.
# Copyright (c) 2017 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils

from zun.common import exception
from zun.common.i18n import _
from zun.pci import devspec


class Whitelist(object):

    """White list class to decide assignable pci devices.

    Not all devices on compute node can be assigned to guest, the
    cloud administrator decides the devices that can be assigned
    based on vendor_id or product_id etc. If no white list specified,
    no device will be assignable.
    """

    def _parse_white_list_from_config(self, whitelists):
        """Parse and validate the pci whitelist from the zun config."""
        specs = []
        for jsonspec in whitelists:
            try:
                dev_spec = jsonutils.loads(jsonspec)
            except ValueError:
                raise exception.PciConfigInvalidWhitelist(
                    reason=_("Invalid entry: '%s'") % jsonspec)
            if isinstance(dev_spec, dict):
                dev_spec = [dev_spec]
            elif not isinstance(dev_spec, list):
                raise exception.PciConfigInvalidWhitelist(
                    reason=_("Invalid entry: '%s'; "
                             "Expecting list or dict") % jsonspec)

            for ds in dev_spec:
                if not isinstance(ds, dict):
                    raise exception.PciConfigInvalidWhitelist(
                        reason=_("Invalid entry: '%s'; "
                                 "Expecting dict") % ds)

                spec = devspec.PciDeviceSpec(ds)
                specs.append(spec)

        return specs

    def __init__(self, whitelist_spec=None):
        """White list constructor

        For example, followed json string specifies that devices whose
        vendor_id is '8086' and product_id is '1520' can be assigned
        to guest.
        '[{"product_id":"1520", "vendor_id":"8086"}]'

        :param whitelist_spec: A json string for a list of dictionaries,
                               each dictionary specifies the pci device
                               properties requirement.
        """
        super(Whitelist, self).__init__()
        if whitelist_spec:
            self.specs = self._parse_white_list_from_config(whitelist_spec)
        else:
            self.specs = []

    def device_assignable(self, dev):
        """Check if a device can be assigned to a guest.

        :param dev: A dictionary describing the device properties
        """
        for spec in self.specs:
            if spec.match(dev):
                return True
        return False

    def get_devspec(self, pci_dev):
        for spec in self.specs:
            if spec.match_pci_obj(pci_dev):
                return spec



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket

from oslo_config import cfg
from oslo_utils import netutils


netconf_opts = [
    cfg.StrOpt('my_ip',
               default=netutils.get_my_ipv4(),
               sample_default='<host_ipv4>',
               help="""
The IP address which the host is using to connect to the management network.

Possible values:

* String with valid IP address. Default is IPv4 address of this host.

Related options:

* docker_remote_api_host
* wsproxy_host
* host_ip
* my_block_storage_ip
"""),
    cfg.HostAddressOpt('host',
                       default=socket.gethostname(),
                       sample_default='<current_hostname>',
                       help="""
Hostname, FQDN or IP address of this host. This can be an opaque identifier.
It is not necessarily a hostname, FQDN, or IP address. However, the node name
must be valid within an AMQP key.

Possible values:

* String with hostname, FQDN or IP address. Default is hostname of this host.
"""),
    cfg.StrOpt("my_block_storage_ip",
               default="$my_ip",
               help="""
The IP address which is used to connect to the block storage network.
Possible values:
* String with valid IP address. Default is IP address of this host.
Related options:
* my_ip - if my_block_storage_ip is not set, then my_ip value is used.
"""),
]


def register_opts(conf):
    conf.register_opts(netconf_opts)


def list_opts():
    return {'DEFAULT': netconf_opts}



# Copyright (c) 2017 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from zun.scheduler import filters

LOG = logging.getLogger(__name__)


class RuntimeFilter(filters.BaseHostFilter):
    """Filter the containers by runtime"""

    run_filter_once_per_request = True

    def host_passes(self, host_state, container, extra_spec):
        if not hasattr(container, 'runtime') or not container.runtime:
            return True

        if container.runtime not in host_state.runtimes:
            LOG.debug("Runtime '%(container_runtime)s' requested. "
                      "%(host_state)s has runtimes: %(host_runtime)s",
                      {'host_state': host_state,
                       'container_runtime': container.runtime,
                       'host_runtime': host_state.runtimes})
            return False
        return True



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mock
from mock import patch

from zun import objects
from zun.tests.unit.api import base as api_base
from zun.tests.unit.db import utils


class TestAvailabilityZoneController(api_base.FunctionalTest):

    @mock.patch('zun.common.policy.enforce')
    @patch('zun.objects.ZunService.list')
    def test_get_all_availability_zones(self,
                                        mock_availability_zone_list,
                                        mock_policy):
        mock_policy.return_value = True
        test_a_zone = utils.get_test_zun_service()
        availability_zones = [objects.ZunService(self.context, **test_a_zone)]
        mock_availability_zone_list.return_value = availability_zones

        response = self.get('/v1/availability_zones')

        mock_availability_zone_list.assert_called_once_with(
            mock.ANY, 1000, None, 'availability_zone', 'asc')
        self.assertEqual(200, response.status_int)
        actual_a_zones = response.json['availability_zones']
        self.assertEqual(1, len(actual_a_zones))
        self.assertEqual(test_a_zone['availability_zone'],
                         actual_a_zones[0].get('availability_zone'))


class TestAvailabilityZonetEnforcement(api_base.FunctionalTest):

    def _common_policy_check(self, rule, func, *arg, **kwarg):
        self.policy.set_rules({rule: 'project_id:non_fake'})
        response = func(*arg, **kwarg)
        self.assertEqual(403, response.status_int)
        self.assertEqual('application/json', response.content_type)
        self.assertTrue(
            "Policy doesn't allow %s to be performed." % rule,
            response.json['errors'][0]['detail'])

    def test_policy_disallow_get_all(self):
        self._common_policy_check(
            'availability_zones:get_all', self.get_json, '/availability_zones',
            expect_errors=True)



# Copyright 2016 Intel.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from glanceclient.common import exceptions as glance_exceptions
from oslo_utils import uuidutils

from zun.common import clients
from zun.common import exception

from oslo_log import log as logging

LOG = logging.getLogger(__name__)


def create_glanceclient(context):
    """Creates glance client object.

        :param context: context to create client object
        :returns: Glance client object
    """
    osc = clients.OpenStackClients(context)
    return osc.glance()


def find_image(context, image_ident, tag):
    matches = find_images(context, image_ident, tag, exact_match=True)
    LOG.debug('Found matches %s ', matches)
    if len(matches) == 0:
        raise exception.ImageNotFound(image=image_ident)
    if len(matches) > 1:
        msg = ("Multiple images exist with same name "
               "%(image_ident)s. Please use the image id "
               "instead.") % {'image_ident': image_ident}
        raise exception.Conflict(msg)
    return matches[0]


def find_images(context, image_ident, tag, exact_match):
    glance = create_glanceclient(context)
    if uuidutils.is_uuid_like(image_ident):
        images = []
        try:
            image = glance.images.get(image_ident)
            if image.container_format == 'docker':
                images.append(image)
        except glance_exceptions.NotFound:
            # ignore exception
            pass
    else:
        filters = {'container_format': 'docker'}
        images = list(glance.images.list(filters=filters))
        if exact_match:
            images = [i for i in images if i.name == image_ident]
        else:
            images = [i for i in images if image_ident in i.name]
        if tag and len(images) > 0:
            if len(images) == 1:
                if images[0].tags and tag not in images[0].tags:
                    del images[0]
            else:
                images = [i for i in images if tag in i.tags]
    return images


def create_image(context, image_name):
    """Create an image."""
    glance = create_glanceclient(context)
    return glance.images.create(name=image_name)


def update_image(context, img_id, disk_format,
                 container_format, tags):
    """Update an image (container format, disk format & tags)"""
    glance = create_glanceclient(context)
    return glance.images.update(img_id, disk_format=disk_format,
                                container_format=container_format, tags=tags)


def upload_image_data(context, img_id, data):
    """Upload an image."""
    LOG.debug('Upload image %s ', img_id)
    glance = create_glanceclient(context)
    return glance.images.upload(img_id, data)


def download_image_in_chunks(context, img_id):
    """Download image in chunks."""
    LOG.debug('Download image %s', img_id)
    glance = create_glanceclient(context)
    return glance.images.data(img_id)


def delete_image(context, img_id):
    """Delete an image."""
    LOG.debug('Delete image %s', img_id)
    glance = create_glanceclient(context)
    return glance.images.delete(img_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zun.common import exception
from zun import objects
from zun.tests import base


class TestNUMA(base.BaseTestCase):
    def test_free_cpus_numa(self):
        obj = objects.NUMATopology(cells=[
            objects.NUMANode(
                id=0, cpuset=set([1, 2]),
                pinned_cpus=set([1])),
            objects.NUMANode(
                id=1, cpuset=set([3, 4]),
                pinned_cpus=set([]))
            ]
        )
        self.assertEqual(set([2]), obj.cells[0].free_cpus)
        self.assertEqual(set([3, 4]), obj.cells[1].free_cpus)

    def test_pinning_logic(self):
        numacell = objects.NUMANode(id=0, cpuset=set([1, 2, 3, 4]),
                                    pinned_cpus=set([1]))
        numacell.pin_cpus(set([2, 3]))
        self.assertEqual(set([4]), numacell.free_cpus)
        self.assertRaises(exception.CPUUnpinningUnknown,
                          numacell.unpin_cpus, set([1, 55]))
        self.assertRaises(exception.CPUPinningInvalid,
                          numacell.pin_cpus, set([1, 4]))
        self.assertRaises(exception.CPUUnpinningInvalid,
                          numacell.unpin_cpus, set([1, 4]))
        numacell.unpin_cpus(set([1, 2, 3]))
        self.assertEqual(set([1, 2, 3, 4]), numacell.free_cpus)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Add init containers uuids to capsule

Revision ID: 54bcb75afb32
Revises: 02134de8e7d3
Create Date: 2018-08-14 15:47:49.127773

"""

# revision identifiers, used by Alembic.
revision = '54bcb75afb32'
down_revision = '02134de8e7d3'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

from zun.db.sqlalchemy import models


def upgrade():
    op.add_column('capsule', sa.Column('init_containers_uuids',
                                       models.JSONEncodedList(),
                                       nullable=True))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from testtools.matchers import HasLength

from zun import objects
from zun.tests.unit.db import base
from zun.tests.unit.db import utils


class TestResourceClassObject(base.DbTestCase):

    def setUp(self):
        super(TestResourceClassObject, self).setUp()
        self.fake_resource = utils.get_test_resource_class()

    def test_get_by_name(self):
        name = self.fake_resource['name']
        with mock.patch.object(self.dbapi, 'get_resource_class',
                               autospec=True) as mock_get_resource_class:
            mock_get_resource_class.return_value = self.fake_resource
            resource = objects.ResourceClass.get_by_name(self.context, name)
            mock_get_resource_class.assert_called_once_with(
                self.context, name)
            self.assertEqual(self.context, resource._context)

    def test_list(self):
        with mock.patch.object(self.dbapi, 'list_resource_classes',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_resource]
            resources = objects.ResourceClass.list(self.context)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(resources, HasLength(1))
            self.assertIsInstance(resources[0], objects.ResourceClass)
            self.assertEqual(self.context, resources[0]._context)

    def test_create(self):
        with mock.patch.object(self.dbapi, 'create_resource_class',
                               autospec=True) as mock_create:
            mock_create.return_value = self.fake_resource
            resource = objects.ResourceClass(
                self.context, **self.fake_resource)
            resource.create(self.context)
            mock_create.assert_called_once_with(
                self.context, self.fake_resource)
            self.assertEqual(self.context, resource._context)

    def test_destroy(self):
        rc_uuid = self.fake_resource['uuid']
        with mock.patch.object(self.dbapi, 'get_resource_class',
                               autospec=True) as mock_get_resource_class:
            mock_get_resource_class.return_value = self.fake_resource
            with mock.patch.object(self.dbapi, 'destroy_resource_class',
                                   autospec=True) as mock_destroy:
                resource = objects.ResourceClass.get_by_uuid(
                    self.context, rc_uuid)
                resource.destroy()
                mock_get_resource_class.assert_called_once_with(
                    self.context, rc_uuid)
                mock_destroy.assert_called_once_with(None, rc_uuid)
                self.assertEqual(self.context, resource._context)

    def test_save(self):
        rc_uuid = self.fake_resource['uuid']
        with mock.patch.object(self.dbapi, 'get_resource_class',
                               autospec=True) as mock_get_resource_class:
            mock_get_resource_class.return_value = self.fake_resource
            with mock.patch.object(self.dbapi, 'update_resource_class',
                                   autospec=True) as mock_update:
                resource = objects.ResourceClass.get_by_uuid(
                    self.context, rc_uuid)
                resource.name = 'MEMORY_MB'
                resource.save()

                mock_get_resource_class.assert_called_once_with(
                    self.context, rc_uuid)
                mock_update.assert_called_once_with(
                    None, rc_uuid,
                    {'name': 'MEMORY_MB'})
                self.assertEqual(self.context, resource._context)

    def test_refresh(self):
        rc_uuid = self.fake_resource['uuid']
        name = self.fake_resource['name']
        new_name = 'MEMORY_MB'
        returns = [dict(self.fake_resource, name=name),
                   dict(self.fake_resource, name=new_name)]
        expected = [mock.call(self.context, rc_uuid),
                    mock.call(self.context, rc_uuid)]
        with mock.patch.object(self.dbapi, 'get_resource_class',
                               side_effect=returns,
                               autospec=True) as mock_get_resource_class:
            resource = objects.ResourceClass.get_by_uuid(
                self.context, rc_uuid)
            self.assertEqual(name, resource.name)
            resource.refresh()
            self.assertEqual(new_name, resource.name)
            self.assertEqual(
                expected, mock_get_resource_class.call_args_list)
            self.assertEqual(self.context, resource._context)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tests for manipulating compute nodes via the DB API"""

from oslo_utils import uuidutils
import six

from zun.common import exception
import zun.conf
from zun.db import api as dbapi
from zun.tests.unit.db import base
from zun.tests.unit.db import utils

CONF = zun.conf.CONF


class DbComputeNodeTestCase(base.DbTestCase):

    def setUp(self):
        super(DbComputeNodeTestCase, self).setUp()

    def test_create_compute_node(self):
        utils.create_test_compute_node(context=self.context)

    def test_create_compute_node_already_exists(self):
        utils.create_test_compute_node(
            context=self.context, uuid='123')
        with self.assertRaisesRegex(exception.ComputeNodeAlreadyExists,
                                    'A compute node with UUID 123.*'):
            utils.create_test_compute_node(
                context=self.context, uuid='123')

    def test_get_compute_node_by_uuid(self):
        node = utils.create_test_compute_node(context=self.context)
        res = dbapi.get_compute_node(
            self.context, node.uuid)
        self.assertEqual(node.uuid, res.uuid)
        self.assertEqual(node.hostname, res.hostname)

    def test_get_compute_node_by_hostname(self):
        node = utils.create_test_compute_node(context=self.context)
        res = dbapi.get_compute_node_by_hostname(
            self.context, node.hostname)
        self.assertEqual(node.uuid, res.uuid)
        self.assertEqual(node.hostname, res.hostname)

    def test_get_compute_node_that_does_not_exist(self):
        self.assertRaises(exception.ComputeNodeNotFound,
                          dbapi.get_compute_node,
                          self.context,
                          uuidutils.generate_uuid())

    def test_list_compute_nodes(self):
        uuids = []
        for i in range(1, 6):
            node = utils.create_test_compute_node(
                uuid=uuidutils.generate_uuid(),
                context=self.context,
                hostname='node' + str(i))
            uuids.append(six.text_type(node['uuid']))
        res = dbapi.list_compute_nodes(self.context)
        res_uuids = [r.uuid for r in res]
        self.assertEqual(sorted(uuids), sorted(res_uuids))

    def test_list_compute_nodes_sorted(self):
        uuids = []
        for i in range(5):
            node = utils.create_test_compute_node(
                uuid=uuidutils.generate_uuid(),
                context=self.context,
                hostname='node' + str(i))
            uuids.append(six.text_type(node.uuid))
        res = dbapi.list_compute_nodes(self.context, sort_key='uuid')
        res_uuids = [r.uuid for r in res]
        self.assertEqual(sorted(uuids), res_uuids)

        self.assertRaises(exception.InvalidParameterValue,
                          dbapi.list_compute_nodes,
                          self.context,
                          sort_key='foo')

    def test_list_compute_nodes_with_filters(self):
        node1 = utils.create_test_compute_node(
            hostname='node-one',
            uuid=uuidutils.generate_uuid(),
            context=self.context)
        node2 = utils.create_test_compute_node(
            hostname='node-two',
            uuid=uuidutils.generate_uuid(),
            context=self.context)

        res = dbapi.list_compute_nodes(
            self.context, filters={'hostname': 'node-one'})
        self.assertEqual([node1.uuid], [r.uuid for r in res])

        res = dbapi.list_compute_nodes(
            self.context, filters={'hostname': 'node-two'})
        self.assertEqual([node2.uuid], [r.uuid for r in res])

        res = dbapi.list_compute_nodes(
            self.context, filters={'hostname': 'bad-node'})
        self.assertEqual([], [r.uuid for r in res])

        res = dbapi.list_compute_nodes(
            self.context,
            filters={'hostname': node1.hostname})
        self.assertEqual([node1.uuid], [r.uuid for r in res])

    def test_destroy_compute_node(self):
        node = utils.create_test_compute_node(context=self.context)
        dbapi.destroy_compute_node(self.context, node.uuid)
        self.assertRaises(exception.ComputeNodeNotFound,
                          dbapi.get_compute_node,
                          self.context, node.uuid)

    def test_destroy_compute_node_by_uuid(self):
        node = utils.create_test_compute_node(context=self.context)
        dbapi.destroy_compute_node(self.context, node.uuid)
        self.assertRaises(exception.ComputeNodeNotFound,
                          dbapi.get_compute_node,
                          self.context, node.uuid)

    def test_destroy_compute_node_that_does_not_exist(self):
        self.assertRaises(exception.ComputeNodeNotFound,
                          dbapi.destroy_compute_node, self.context,
                          uuidutils.generate_uuid())

    def test_update_compute_node(self):
        node = utils.create_test_compute_node(context=self.context)
        old_hostname = node.hostname
        new_hostname = 'new-hostname'
        self.assertNotEqual(old_hostname, new_hostname)

        res = dbapi.update_compute_node(
            self.context, node.uuid, {'hostname': new_hostname})
        self.assertEqual(new_hostname, res.hostname)

    def test_update_compute_node_not_found(self):
        node_uuid = uuidutils.generate_uuid()
        new_hostname = 'new-hostname'
        self.assertRaises(exception.ComputeNodeNotFound,
                          dbapi.update_compute_node, self.context,
                          node_uuid, {'hostname': new_hostname})

    def test_update_compute_node_uuid(self):
        node = utils.create_test_compute_node(context=self.context)
        self.assertRaises(exception.InvalidParameterValue,
                          dbapi.update_compute_node, self.context,
                          node.uuid, {'uuid': ''})



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re


def _quote_meta(s):
    special_chars = frozenset("()[]{}?*+|^$\\.-#&~")
    escape = lambda c: r'\{}'.format(c) if c in special_chars else c
    sp = (escape(c) for c in s)
    return r''.join(sp)


def match(regexp):
    return re.compile(regexp)


def literal(s):
    return match(_quote_meta(s))


def expression(*res):
    return match(r''.join(r.pattern for r in res))


def optional(*res):
    return match(r'{}?'.format(group(expression(*res)).pattern))


def repeated(*res):
    return match(r'{}+'.format(group(expression(*res)).pattern))


def group(*res):
    return match(r'(?:{})'.format(expression(*res).pattern))


def capture(*res):
    return match(r'({})'.format(expression(*res).pattern))


def anchored(*res):
    return match(r'^{}$'.format(expression(*res).pattern))


class ImageRegexps(object):
    ALPHA_NUMERIC_REGEXP = match(r'[a-z0-9]+')
    SEPARATOR_REGEXP = match(r'(?:[._]|__|[-]*)')
    NAME_COMPONENT_REGEXP = expression(
        ALPHA_NUMERIC_REGEXP,
        optional(repeated(SEPARATOR_REGEXP, ALPHA_NUMERIC_REGEXP))
    )
    HOSTNAME_COMPONENT_REGEXP = match(
        r'(?:[a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9-]*[a-zA-Z0-9])')
    HOSTNAME_REGEXP = expression(
        HOSTNAME_COMPONENT_REGEXP,
        optional(repeated(literal(r'.'), HOSTNAME_COMPONENT_REGEXP)),
        optional(literal(r':'), match(r'[0-9]+'))
    )
    TAG_REGEXP = match(r'[\w][\w.-]{0,127}')
    ANCHORED_TAG_REGEXP = anchored(TAG_REGEXP)
    DIGEST_REGEXP = match(r'[a-zA-Z0-9-_+.]+:[a-fA-F0-9]+')
    ANCHORED_DIGEST_REGEXP = anchored(DIGEST_REGEXP)
    NAME_REGEXP = expression(
        optional(HOSTNAME_REGEXP, literal(r'/')),
        NAME_COMPONENT_REGEXP,
        optional(repeated(literal(r'/'), NAME_COMPONENT_REGEXP))
    )
    ANCHORED_NAME_REGEXP = anchored(
        optional(capture(HOSTNAME_REGEXP), literal(r'/')),
        capture(NAME_COMPONENT_REGEXP,
                optional(repeated(literal(r'/'), NAME_COMPONENT_REGEXP)))
    )
    REFERENCE_REGEXP = anchored(
        capture(NAME_REGEXP),
        optional(literal(r':'), capture(TAG_REGEXP)),
        optional(literal(r'@'), capture(DIGEST_REGEXP))
    )


class DigestRegexps(object):
    DIGEST_REGEXP = match(r'[a-zA-Z0-9-_+.]+:[a-fA-F0-9]+')
    DIGEST_REGEXP_ANCHORED = anchored(DIGEST_REGEXP)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from cinderclient import exceptions as cinder_exception
from os_brick import exception as os_brick_exception
from oslo_serialization import jsonutils

import zun.conf
from zun.tests import base
from zun.volume import cinder_workflow


CONF = zun.conf.CONF


class CinderWorkflowTestCase(base.TestCase):

    def setUp(self):
        super(CinderWorkflowTestCase, self).setUp()
        self.fake_volume_id = 'fake-volume-id-1'
        self.fake_conn_prprts = {
            'ip': '10.3.4.5',
            'host': 'fakehost1'
        }
        self.fake_device_info = {
            'path': '/foo'
        }
        self.fake_conn_info = {
            'driver_volume_type': 'fake',
            'data': {},
        }

    @mock.patch('zun.volume.cinder_workflow.get_volume_connector')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector_properties')
    @mock.patch('zun.volume.cinder_api.CinderAPI')
    def test_attach_volume(self,
                           mock_cinder_api_cls,
                           mock_get_connector_prprts,
                           mock_get_volume_connector):
        mock_cinder_api, mock_connector = self._test_attach_volume(
            mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector)

        mock_cinder_api.reserve_volume.assert_called_once_with(
            self.fake_volume_id)
        mock_cinder_api.initialize_connection.assert_called_once_with(
            self.fake_volume_id, self.fake_conn_prprts)
        mock_connector.connect_volume.assert_called_once_with(
            self.fake_conn_info['data'])
        mock_cinder_api.attach.assert_called_once_with(
            volume_id=self.fake_volume_id,
            mountpoint=self.fake_device_info['path'],
            hostname=CONF.host,
            container_uuid='123')
        mock_connector.disconnect_volume.assert_not_called()
        mock_cinder_api.terminate_connection.assert_not_called()
        mock_cinder_api.detach.assert_not_called()
        mock_cinder_api.unreserve_volume.assert_not_called()

    @mock.patch('zun.volume.cinder_workflow.get_volume_connector')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector_properties')
    @mock.patch('zun.volume.cinder_api.CinderAPI')
    def test_attach_volume_fail_reserve_volume(
            self, mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector):
        mock_cinder_api, mock_connector = self._test_attach_volume(
            mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector, fail_reserve=True)

        mock_cinder_api.reserve_volume.assert_called_once_with(
            self.fake_volume_id)
        mock_cinder_api.initialize_connection.assert_not_called()
        mock_connector.connect_volume.assert_not_called()
        mock_cinder_api.attach.assert_not_called()
        mock_connector.disconnect_volume.assert_not_called()
        mock_cinder_api.terminate_connection.assert_not_called()
        mock_cinder_api.detach.assert_not_called()
        mock_cinder_api.unreserve_volume.assert_called_once_with(
            self.fake_volume_id)

    @mock.patch('zun.volume.cinder_workflow.get_volume_connector')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector_properties')
    @mock.patch('zun.volume.cinder_api.CinderAPI')
    def test_attach_volume_fail_initialize_connection(
            self, mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector):
        mock_cinder_api, mock_connector = self._test_attach_volume(
            mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector, fail_init=True)

        mock_cinder_api.reserve_volume.assert_called_once_with(
            self.fake_volume_id)
        mock_cinder_api.initialize_connection.assert_called_once_with(
            self.fake_volume_id, self.fake_conn_prprts)
        mock_connector.connect_volume.assert_not_called()
        mock_cinder_api.attach.assert_not_called()
        mock_connector.disconnect_volume.assert_not_called()
        mock_cinder_api.terminate_connection.assert_not_called()
        mock_cinder_api.detach.assert_not_called()
        mock_cinder_api.unreserve_volume.assert_called_once_with(
            self.fake_volume_id)

    @mock.patch('zun.volume.cinder_workflow.get_volume_connector')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector_properties')
    @mock.patch('zun.volume.cinder_api.CinderAPI')
    def test_attach_volume_fail_connect_volume(
            self, mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector):
        mock_cinder_api, mock_connector = self._test_attach_volume(
            mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector, fail_connect=True)

        mock_cinder_api.reserve_volume.assert_called_once_with(
            self.fake_volume_id)
        mock_cinder_api.initialize_connection.assert_called_once_with(
            self.fake_volume_id, self.fake_conn_prprts)
        mock_connector.connect_volume.assert_called_once_with(
            self.fake_conn_info['data'])
        mock_cinder_api.attach.assert_not_called()
        mock_connector.disconnect_volume.assert_not_called()
        mock_cinder_api.terminate_connection.assert_called_once_with(
            self.fake_volume_id, self.fake_conn_prprts)
        mock_cinder_api.detach.assert_not_called()
        mock_cinder_api.unreserve_volume.assert_called_once_with(
            self.fake_volume_id)

    @mock.patch('zun.volume.cinder_workflow.get_volume_connector')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector_properties')
    @mock.patch('zun.volume.cinder_api.CinderAPI')
    def test_attach_volume_fail_attach(self,
                                       mock_cinder_api_cls,
                                       mock_get_connector_prprts,
                                       mock_get_volume_connector):
        mock_cinder_api, mock_connector = self._test_attach_volume(
            mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector, fail_attach=True)

        mock_cinder_api.reserve_volume.assert_called_once_with(
            self.fake_volume_id)
        mock_cinder_api.initialize_connection.assert_called_once_with(
            self.fake_volume_id, self.fake_conn_prprts)
        mock_connector.connect_volume.assert_called_once_with(
            self.fake_conn_info['data'])
        mock_cinder_api.attach.assert_called_once_with(
            volume_id=self.fake_volume_id,
            mountpoint=self.fake_device_info['path'],
            hostname=CONF.host,
            container_uuid='123')
        mock_connector.disconnect_volume.assert_called_once_with(
            self.fake_conn_info['data'], None)
        mock_cinder_api.terminate_connection.assert_called_once_with(
            self.fake_volume_id, self.fake_conn_prprts)
        mock_cinder_api.detach.assert_called_once_with(
            mock.ANY)
        mock_cinder_api.unreserve_volume.assert_called_once_with(
            self.fake_volume_id)

    def _test_attach_volume(self,
                            mock_cinder_api_cls,
                            mock_get_connector_prprts,
                            mock_get_volume_connector,
                            fail_reserve=False, fail_init=False,
                            fail_connect=False, fail_attach=False):
        volume = mock.MagicMock()
        volume.cinder_volume_id = self.fake_volume_id
        volume.container_uuid = '123'
        mock_cinder_api = mock.MagicMock()
        mock_cinder_api_cls.return_value = mock_cinder_api
        mock_connector = mock.MagicMock()
        mock_get_connector_prprts.return_value = self.fake_conn_prprts
        mock_get_volume_connector.return_value = mock_connector
        mock_cinder_api.initialize_connection.return_value = \
            self.fake_conn_info
        mock_connector.connect_volume.return_value = self.fake_device_info
        cinder = cinder_workflow.CinderWorkflow(self.context)

        if fail_reserve:
            mock_cinder_api.reserve_volume.side_effect = \
                cinder_exception.ClientException(400)
            self.assertRaises(cinder_exception.ClientException,
                              cinder.attach_volume, volume)
        elif fail_init:
            mock_cinder_api.initialize_connection.side_effect = \
                cinder_exception.ClientException(400)
            self.assertRaises(cinder_exception.ClientException,
                              cinder.attach_volume, volume)
        elif fail_connect:
            mock_connector.connect_volume.side_effect = \
                os_brick_exception.BrickException()
            self.assertRaises(os_brick_exception.BrickException,
                              cinder.attach_volume, volume)
        elif fail_attach:
            mock_cinder_api.attach.side_effect = \
                cinder_exception.ClientException(400)
            self.assertRaises(cinder_exception.ClientException,
                              cinder.attach_volume, volume)
        else:
            device_path = cinder.attach_volume(volume)
            self.assertEqual('/foo', device_path)

        return mock_cinder_api, mock_connector

    @mock.patch('zun.volume.cinder_workflow.CinderWorkflow.'
                '_volume_connection_keep')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector_properties')
    @mock.patch('zun.volume.cinder_api.CinderAPI')
    def test_detach_volume(self,
                           mock_cinder_api_cls,
                           mock_get_connector_prprts,
                           mock_get_volume_connector,
                           mock_connection_keep):
        volume = mock.MagicMock()
        volume.cinder_volume_id = self.fake_volume_id
        volume.connection_info = jsonutils.dumps(self.fake_conn_info)
        mock_cinder_api = mock.MagicMock()
        mock_cinder_api_cls.return_value = mock_cinder_api
        mock_connector = mock.MagicMock()
        mock_get_connector_prprts.return_value = self.fake_conn_prprts
        mock_get_volume_connector.return_value = mock_connector
        mock_connection_keep.return_value = False

        cinder = cinder_workflow.CinderWorkflow(self.context)
        cinder.detach_volume(self.context, volume)

        mock_cinder_api.begin_detaching.assert_called_once_with(
            self.fake_volume_id)
        mock_connector.disconnect_volume.assert_called_once_with(
            self.fake_conn_info['data'], None)
        mock_cinder_api.terminate_connection.assert_called_once_with(
            self.fake_volume_id, self.fake_conn_prprts)
        mock_cinder_api.detach.assert_called_once_with(volume)
        mock_cinder_api.roll_detaching.assert_not_called()

    @mock.patch('zun.volume.cinder_workflow.CinderWorkflow.'
                '_volume_connection_keep')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector')
    @mock.patch('zun.volume.cinder_workflow.get_volume_connector_properties')
    @mock.patch('zun.volume.cinder_api.CinderAPI')
    def test_detach_volume_fail_disconnect(
            self, mock_cinder_api_cls, mock_get_connector_prprts,
            mock_get_volume_connector, mock_connection_keep):
        volume = mock.MagicMock()
        volume.cinder_volume_id = self.fake_volume_id
        volume.connection_info = jsonutils.dumps(self.fake_conn_info)
        mock_cinder_api = mock.MagicMock()
        mock_cinder_api_cls.return_value = mock_cinder_api
        mock_connector = mock.MagicMock()
        mock_get_connector_prprts.return_value = self.fake_conn_prprts
        mock_get_volume_connector.return_value = mock_connector
        mock_connection_keep.return_value = False
        mock_connector.disconnect_volume.side_effect = \
            os_brick_exception.BrickException()

        cinder = cinder_workflow.CinderWorkflow(self.context)
        self.assertRaises(os_brick_exception.BrickException,
                          cinder.detach_volume, self.context, volume)

        mock_cinder_api.begin_detaching.assert_called_once_with(
            self.fake_volume_id)
        mock_connector.disconnect_volume.assert_called_once_with(
            self.fake_conn_info['data'], None)
        mock_cinder_api.terminate_connection.assert_not_called()
        mock_cinder_api.detach.assert_not_called()
        mock_cinder_api.roll_detaching.assert_called_once_with(
            self.fake_volume_id)

    @mock.patch('zun.volume.cinder_api.CinderAPI')
    def test_delete_volume(self,
                           mock_cinder_api_cls):
        volume = mock.MagicMock()
        volume.cinder_volume_id = self.fake_volume_id
        volume.connection_info = jsonutils.dumps(self.fake_conn_info)
        mock_cinder_api = mock.MagicMock()
        mock_cinder_api_cls.return_value = mock_cinder_api

        cinder = cinder_workflow.CinderWorkflow(self.context)
        cinder.delete_volume(volume)

        mock_cinder_api.delete_volume.assert_called_once_with(
            self.fake_volume_id)



# Copyright (c) 2017 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log.log import logging

from zun.api import servicegroup
from zun.scheduler import filters

LOG = logging.getLogger(__name__)


class ComputeFilter(filters.BaseHostFilter):
    """Filter on active Compute nodes"""

    def __init__(self):
        self.servicegroup_api = servicegroup.ServiceGroup()
        super(ComputeFilter, self).__init__()

    # Host state does not change within a request
    run_filter_once_per_request = True

    def host_passes(self, host_state, container, extra_spec):
        """Returns True for only active compute nodes"""
        service = host_state.service
        if service.disabled:
            LOG.debug('%(host_state)s is disabled, reason: %(reason)s',
                      {'host_state': host_state.hostname,
                       'reason': service.disabled_reason or 'Unknow'})
            return False
        else:
            if not self.servicegroup_api.service_is_up(service):
                LOG.warning('%(host_state)s has not been heard from in '
                            'a while', {'host_state': host_state.hostname})
                return False
        return True



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zun.objects import compute_node
from zun.objects import container
from zun.objects import container_action
from zun.objects import container_pci_requests
from zun.objects import exec_instance
from zun.objects import image
from zun.objects import network
from zun.objects import numa
from zun.objects import pci_device
from zun.objects import pci_device_pool
from zun.objects import quota
from zun.objects import quota_class
from zun.objects import registry
from zun.objects import request_group
from zun.objects import resource_class
from zun.objects import resource_provider
from zun.objects import volume
from zun.objects import volume_mapping
from zun.objects import zun_service


Container = container.Container
Volume = volume.Volume
VolumeMapping = volume_mapping.VolumeMapping
ZunService = zun_service.ZunService
Image = image.Image
Network = network.Network
NUMANode = numa.NUMANode
NUMATopology = numa.NUMATopology
ResourceProvider = resource_provider.ResourceProvider
ResourceClass = resource_class.ResourceClass
ComputeNode = compute_node.ComputeNode
Capsule = container.Capsule
CapsuleContainer = container.CapsuleContainer
CapsuleInitContainer = container.CapsuleInitContainer
PciDevice = pci_device.PciDevice
PciDevicePool = pci_device_pool.PciDevicePool
Quota = quota.Quota
QuotaClass = quota_class.QuotaClass
ContainerPCIRequest = container_pci_requests.ContainerPCIRequest
ContainerPCIRequests = container_pci_requests.ContainerPCIRequests
ContainerAction = container_action.ContainerAction
ContainerActionEvent = container_action.ContainerActionEvent
ExecInstance = exec_instance.ExecInstance
Registry = registry.Registry
RequestGroup = request_group.RequestGroup

__all__ = (
    'Container',
    'VolumeMapping',
    'ZunService',
    'Image',
    'Network',
    'ResourceProvider',
    'ResourceClass',
    'NUMANode',
    'NUMATopology',
    'ComputeNode',
    'Capsule',
    'CapsuleContainer',
    'CapsuleInitContainer',
    'PciDevice',
    'PciDevicePool',
    'Quota',
    'QuotaClass',
    'ContainerPCIRequest',
    'ContainerPCIRequests',
    'ContainerAction',
    'ContainerActionEvent',
    'ExecInstance',
    'Registry',
    'RequestGroup',
)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add_contents_to_volume_mapping_table

Revision ID: a9c9fb54274a
Revises: bc56b9932dd9
Create Date: 2018-08-10 02:49:27.524151

"""

# revision identifiers, used by Alembic.
revision = 'a9c9fb54274a'
down_revision = 'bc56b9932dd9'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def MediumText():
    return sa.Text().with_variant(sa.dialects.mysql.MEDIUMTEXT(), 'mysql')


def upgrade():
    op.add_column('volume_mapping',
                  sa.Column('contents', MediumText(), nullable=True))
    op.alter_column('volume_mapping', 'volume_id',
                    existing_type=sa.String(36),
                    nullable=True)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Starter script for zun-db-manage."""

from oslo_config import cfg

from zun.db import migration


CONF = cfg.CONF


def do_version():
    print('Current DB revision is %s' % migration.version())


def do_upgrade():
    migration.upgrade(CONF.command.revision)


def do_stamp():
    migration.stamp(CONF.command.revision)


def do_revision():
    migration.revision(message=CONF.command.message,
                       autogenerate=CONF.command.autogenerate)


def add_command_parsers(subparsers):
    parser = subparsers.add_parser('version')
    parser.set_defaults(func=do_version)

    parser = subparsers.add_parser('upgrade')
    parser.add_argument('revision', nargs='?')
    parser.set_defaults(func=do_upgrade)

    parser = subparsers.add_parser('stamp')
    parser.add_argument('revision')
    parser.set_defaults(func=do_stamp)

    parser = subparsers.add_parser('revision')
    parser.add_argument('-m', '--message')
    parser.add_argument('--autogenerate', action='store_true')
    parser.set_defaults(func=do_revision)


def main():
    command_opt = cfg.SubCommandOpt('command',
                                    title='Command',
                                    help='Available commands',
                                    handler=add_command_parsers)
    CONF.register_cli_opt(command_opt)

    CONF(project='zun')
    CONF.command.func()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Scheduler host filters
"""
from zun.scheduler import base_filters


class BaseHostFilter(base_filters.BaseFilter):
    """Base class for host filters."""
    def _filter_one(self, obj, filter_properties, extra_spec):
        """Return True if the object passes the filter, otherwise False."""
        return self.host_passes(obj, filter_properties, extra_spec)

    def host_passes(self, host_state, filter_properties, extra_spec):
        """Return True if the HostState passes the filter,otherwise False.

        Override this in a subclass.
        """
        raise NotImplementedError()


class HostFilterHandler(base_filters.BaseFilterHandler):
    def __init__(self):
        super(HostFilterHandler, self).__init__(BaseHostFilter)


def all_filters():
    """Return a list of filter classes found in this directory.

    This method is used as the default for available scheduler filters
    and should return a list of all filter classes available.
    """

    return HostFilterHandler().get_all_classes()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from oslo_serialization import jsonutils

from zun.container.docker import utils as docker_utils
from zun.tests.unit.container import base


class TestDockerHTTPClient(base.DriverTestCase):

    def setUp(self):
        super(TestDockerHTTPClient, self).setUp()
        self.client = docker_utils.DockerHTTPClient()

    @mock.patch('tarfile.open')
    def test_read_tar_image(self, mock_open):
        fake_image = {'path': 'fake-path'}
        mock_context_manager = mock.MagicMock()
        mock_open.return_value = mock_context_manager
        mock_file = mock.MagicMock()
        mock_context_manager.__enter__.return_value = mock_file
        mock_data = [{"Config": "fake_config",
                      "RepoTags": ["cirros:latest"],
                      "Layers": ["fake_layer", "fake_layer2"]}]
        mock_file.extractfile.return_value.read.return_value = \
            jsonutils.dumps(mock_data, separators=(',', ':'))

        self.client.read_tar_image(fake_image)
        self.assertEqual('cirros', fake_image['repo'])
        self.assertEqual('latest', fake_image['tag'])

    @mock.patch('tarfile.open')
    def test_read_tar_image_multi_tags(self, mock_open):
        fake_image = {'path': 'fake-path'}
        mock_context_manager = mock.MagicMock()
        mock_open.return_value = mock_context_manager
        mock_file = mock.MagicMock()
        mock_context_manager.__enter__.return_value = mock_file
        mock_data = [{"Config": "fake_config",
                      "RepoTags": ["cirros:latest", "cirros:0.3.4"],
                      "Layers": ["fake_layer", "fake_layer2"]}]
        mock_file.extractfile.return_value.read.return_value = \
            jsonutils.dumps(mock_data, separators=(',', ':'))

        self.client.read_tar_image(fake_image)
        self.assertEqual('cirros', fake_image['repo'])
        self.assertEqual('latest', fake_image['tag'])

    @mock.patch('tarfile.open')
    def test_read_tar_image_no_repotag(self, mock_open):
        fake_image = {'path': 'fake-path'}
        mock_context_manager = mock.MagicMock()
        mock_open.return_value = mock_context_manager
        mock_file = mock.MagicMock()
        mock_context_manager.__enter__.return_value = mock_file
        mock_data = [{"Config": "fake_config",
                      "RepoTags": "",
                      "Layers": ["fake_layer", "fake_layer2"]}]
        mock_file.extractfile.return_value.read.return_value = \
            jsonutils.dumps(mock_data, separators=(',', ':'))

        self.client.read_tar_image(fake_image)
        self.assertEqual('fake_config', fake_image['repo'])
        self.assertEqual('', fake_image['tag'])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""add container_type to container

Revision ID: d73b72ab7cc6
Revises: 1bc34e18180b
Create Date: 2019-02-12 04:34:30.993517

"""

# revision identifiers, used by Alembic.
revision = 'd73b72ab7cc6'
down_revision = '1bc34e18180b'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

from zun.common import consts


def upgrade():
    op.add_column('container',
                  sa.Column('container_type', sa.Integer(), index=True,
                            default=consts.TYPE_CONTAINER,
                            server_default=str(consts.TYPE_CONTAINER)))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Handles all requests relating to compute resources (e.g. containers,
networking and storage of containers, and compute hosts on which they run)."""

from oslo_log import log as logging

from zun.common import consts
from zun.common import exception
from zun.common.i18n import _
from zun.common import profiler
from zun.compute import container_actions
from zun.compute import rpcapi
import zun.conf
from zun import objects
from zun.scheduler.client import query as scheduler_client


CONF = zun.conf.CONF
LOG = logging.getLogger(__name__)


@profiler.trace_cls("rpc")
class API(object):
    """API for interacting with the compute manager."""

    def __init__(self, context):
        self.rpcapi = rpcapi.API(context=context)
        self.scheduler_client = scheduler_client.SchedulerClient()
        super(API, self).__init__()

    def _record_action_start(self, context, container, action):
        objects.ContainerAction.action_start(context, container.uuid,
                                             action, want_result=False)

    def container_create(self, context, new_container, extra_spec,
                         requested_networks, requested_volumes, run,
                         pci_requests=None):
        try:
            host_state = self._schedule_container(context, new_container,
                                                  extra_spec)
        except exception.NoValidHost:
            new_container.status = consts.ERROR
            new_container.status_reason = _(
                "There are not enough hosts available.")
            new_container.save(context)
            return
        except Exception:
            new_container.status = consts.ERROR
            new_container.status_reason = _("Unexpected exception occurred.")
            new_container.save(context)
            raise

        # NOTE(mkrai): Intent here is to check the existence of image
        # before proceeding to create container. If image is not found,
        # container create will fail with 400 status.
        if CONF.api.enable_image_validation:
            try:
                images = self.rpcapi.image_search(
                    context, new_container.image,
                    new_container.image_driver, True, new_container.registry,
                    host_state['host'])
                if not images:
                    raise exception.ImageNotFound(image=new_container.image)
                if len(images) > 1:
                    raise exception.Conflict('Multiple images exist with same '
                                             'name. Please use the container '
                                             'uuid instead.')
            except exception.OperationNotSupported:
                LOG.info("Skip validation since search is not supported for "
                         "image '%(image)s' and image driver '%(driver)s'.",
                         {'image': new_container.image,
                          'driver': new_container.image_driver})
            except exception.ReferenceInvalidFormat:
                raise exception.InvalidValue(_("The format of image name '%s' "
                                               "is invalid.")
                                             % new_container.image)
            except Exception as e:
                LOG.warning("Skip validation since image search failed with "
                            "unexpected exception: %s", str(e))

        self._record_action_start(context, new_container,
                                  container_actions.CREATE)
        self.rpcapi.container_create(context, host_state['host'],
                                     new_container, host_state['limits'],
                                     requested_networks, requested_volumes,
                                     run, pci_requests)

    def _schedule_container(self, context, new_container, extra_spec):
        dests = self.scheduler_client.select_destinations(context,
                                                          [new_container],
                                                          extra_spec)
        return dests[0]

    def container_delete(self, context, container, *args):
        self._record_action_start(context, container, container_actions.DELETE)
        return self.rpcapi.container_delete(context, container, *args)

    def container_show(self, context, container):
        return self.rpcapi.container_show(context, container)

    def container_rebuild(self, context, container, run):
        self._record_action_start(context, container,
                                  container_actions.REBUILD)
        return self.rpcapi.container_rebuild(context, container, run)

    def container_reboot(self, context, container, *args):
        self._record_action_start(context, container, container_actions.REBOOT)
        return self.rpcapi.container_reboot(context, container, *args)

    def container_stop(self, context, container, *args):
        self._record_action_start(context, container, container_actions.STOP)
        return self.rpcapi.container_stop(context, container, *args)

    def container_start(self, context, container):
        self._record_action_start(context, container, container_actions.START)
        return self.rpcapi.container_start(context, container)

    def container_pause(self, context, container):
        self._record_action_start(context, container, container_actions.PAUSE)
        return self.rpcapi.container_pause(context, container)

    def container_unpause(self, context, container):
        self._record_action_start(context, container,
                                  container_actions.UNPAUSE)
        return self.rpcapi.container_unpause(context, container)

    def container_logs(self, context, container, stdout, stderr,
                       timestamps, tail, since):
        return self.rpcapi.container_logs(context, container, stdout, stderr,
                                          timestamps, tail, since)

    def container_exec(self, context, container, *args):
        data = self.rpcapi.container_exec(context, container, *args)
        token = data.pop('token', None)
        exec_id = data.get('exec_id')
        if token:
            data['proxy_url'] = '%s?token=%s&uuid=%s&exec_id=%s' % (
                CONF.websocket_proxy.base_url, token, container.uuid, exec_id)
        else:
            data['proxy_url'] = None
        return data

    def container_exec_resize(self, context, container, *args):
        return self.rpcapi.container_exec_resize(context, container, *args)

    def container_kill(self, context, container, *args):
        self._record_action_start(context, container, container_actions.KILL)
        return self.rpcapi.container_kill(context, container, *args)

    def container_update(self, context, container, *args):
        return self.rpcapi.container_update(context, container, *args)

    def container_attach(self, context, container):
        token = self.rpcapi.container_attach(context, container)
        access_url = '%s?token=%s&uuid=%s' % (
            CONF.websocket_proxy.base_url, token, container.uuid)
        return access_url

    def container_resize(self, context, container, *args):
        return self.rpcapi.container_resize(context, container, *args)

    def container_top(self, context, container, *args):
        return self.rpcapi.container_top(context, container, *args)

    def container_get_archive(self, context, container, *args):
        return self.rpcapi.container_get_archive(context, container, *args)

    def add_security_group(self, context, container, *args):
        self._record_action_start(context, container,
                                  container_actions.ADD_SECURITY_GROUP)
        return self.rpcapi.add_security_group(context, container, *args)

    def remove_security_group(self, context, container, *args):
        self._record_action_start(context, container,
                                  container_actions.REMOVE_SECURITY_GROUP)
        return self.rpcapi.remove_security_group(context, container, *args)

    def container_put_archive(self, context, container, *args):
        return self.rpcapi.container_put_archive(context, container, *args)

    def container_stats(self, context, container):
        return self.rpcapi.container_stats(context, container)

    def container_commit(self, context, container, *args):
        self._record_action_start(context, container, container_actions.COMMIT)
        return self.rpcapi.container_commit(context, container, *args)

    def image_delete(self, context, image):
        return self.rpcapi.image_delete(context, image, image.host)

    def image_pull(self, context, image):
        return self.rpcapi.image_pull(context, image, image.host)

    def image_search(self, context, image, image_driver, exact_match, *args):
        return self.rpcapi.image_search(context, image, image_driver,
                                        exact_match, *args)

    def network_detach(self, context, container, *args):
        self._record_action_start(context, container,
                                  container_actions.NETWORK_DETACH)
        return self.rpcapi.network_detach(context, container, *args)

    def network_attach(self, context, container, *args):
        self._record_action_start(context, container,
                                  container_actions.NETWORK_ATTACH)
        return self.rpcapi.network_attach(context, container, *args)

    def network_create(self, context, *args):
        return self.rpcapi.network_create(context, *args)

    def network_delete(self, context, *args):
        return self.rpcapi.network_delete(context, *args)

    def resize_container(self, context, container, *args):
        return self.rpcapi.resize_container(context, container, *args)



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


"""add registry table

Revision ID: 5ffc1cabe6b4
Revises: 21fa080c818a
Create Date: 2019-01-04 02:22:45.889795

"""

# revision identifiers, used by Alembic.
revision = '5ffc1cabe6b4'
down_revision = '21fa080c818a'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table(
        'registry',
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('project_id', sa.String(length=255), nullable=True),
        sa.Column('user_id', sa.String(length=255), nullable=True),
        sa.Column('uuid', sa.String(length=36), nullable=True),
        sa.Column('name', sa.String(length=255), nullable=True),
        sa.Column('domain', sa.String(length=255), nullable=False),
        sa.Column('username', sa.String(length=255), nullable=True),
        sa.Column('password', sa.String(length=255), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('uuid', name='uniq_registry0uuid'),
        mysql_charset='utf8',
        mysql_engine='InnoDB'
    )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils
import six

from zun.common import exception
import zun.conf
from zun.db import api as dbapi
from zun.tests.unit.db import base
from zun.tests.unit.db import utils

CONF = zun.conf.CONF


class DbNetworkTestCase(base.DbTestCase):

    def setUp(self):
        super(DbNetworkTestCase, self).setUp()

    def test_create_network(self):
        utils.create_test_network(context=self.context)

    def test_create_network_already_exists(self):
        utils.create_test_network(context=self.context,
                                  uuid='123', neutron_net_id='456')
        with self.assertRaisesRegex(exception.NetworkAlreadyExists,
                                    'A network with UUID 123.*'):
            utils.create_test_network(context=self.context, uuid='123')
        with self.assertRaisesRegex(exception.NetworkAlreadyExists,
                                    'A network with neutron_net_id 456.*'):
            utils.create_test_network(context=self.context,
                                      neutron_net_id='456')

    def test_list_networks(self):
        uuids = []
        for i in range(1, 6):
            network = utils.create_test_network(
                uuid=uuidutils.generate_uuid(),
                context=self.context,
                neutron_net_id=uuidutils.generate_uuid(),
                name='network' + str(i))
            uuids.append(six.text_type(network['uuid']))
        res = dbapi.list_networks(self.context)
        res_uuids = [r.uuid for r in res]
        self.assertEqual(sorted(uuids), sorted(res_uuids))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import docker
from tempest.lib import decorators

from zun.tests.fullstack import base
from zun.tests.fullstack import utils


class TestContainer(base.BaseFullStackTestCase):

    def setUp(self):
        super(TestContainer, self).setUp()
        self.containers = []

    def tearDown(self):
        containers = self.zun.containers.list()
        for c in containers:
            if c.uuid in self.containers:
                self.zun.containers.delete(c.uuid, stop=True)
                self.ensure_container_deleted(c.uuid)

        super(TestContainer, self).tearDown()

    @decorators.idempotent_id('039fc590-7711-4b87-86bf-fe9048c3feb9')
    def test_run_container(self):
        self._run_container()

    def _run_container(self, **kwargs):
        if not kwargs.get('image'):
            kwargs['image'] = 'cirros:latest'
            kwargs['command'] = ['sleep', '100000']

        kwargs.setdefault('cpu', 0.1)
        kwargs.setdefault('memory', 128)

        container = self.zun.containers.run(**kwargs)
        self.containers.append(container.uuid)
        # Wait for container to started
        self.ensure_container_in_desired_state(container.uuid, 'Running')

        # Assert the container is started
        container = self.zun.containers.get(container.uuid)
        self.assertEqual('Running', container.status)
        self.assertEqual('Running', self._get_state_in_docker(container))
        return container

    def _create_container(self, **kwargs):
        if not kwargs.get('image'):
            kwargs['image'] = 'cirros:latest'
            kwargs['command'] = ['sleep', '100000']

        kwargs.setdefault('cpu', 0.1)
        kwargs.setdefault('memory', 128)

        container = self.zun.containers.create(**kwargs)
        self.containers.append(container.uuid)
        # Wait for container to finish creation
        self.ensure_container_in_desired_state(container.uuid, 'Created')

        # Assert the container is created
        container = self.zun.containers.get(container.uuid)
        self.assertEqual('Created', container.status)
        self.assertEqual('Created', self._get_state_in_docker(container))
        return container

    @decorators.idempotent_id('8c6f0844-1a5c-4bf4-81d5-38dccb2c2b25')
    def test_delete_container(self):
        container = self._create_container()
        self.zun.containers.delete(container.uuid)
        self.ensure_container_deleted(container.uuid)
        self.assertRaises(docker.errors.NotFound,
                          self._get_container_in_docker, container)

    @decorators.idempotent_id('6f7a4d0f-273a-4321-ba14-246c6ea387a1')
    def test_run_container_with_environment(self):
        container = self._run_container(
            environment={'key1': 'env1', 'key2': 'env2'})

        docker_container = self._get_container_in_docker(container)
        env = docker_container['Config']['Env']
        self.assertTrue('key1=env1' in env)
        self.assertTrue('key2=env2' in env)

    @decorators.idempotent_id('25e19899-d450-4d6b-9dbd-160f9c557877')
    def test_run_container_with_labels(self):
        container = self._run_container(
            labels={'key1': 'label1', 'key2': 'label2'})

        docker_container = self._get_container_in_docker(container)
        labels = docker_container['Config']['Labels']
        self.assertEqual({'key1': 'label1', 'key2': 'label2'}, labels)

    @decorators.idempotent_id('8a920b08-32df-448e-ab53-b640611ac769')
    def test_run_container_with_restart_policy(self):
        container = self._run_container(restart_policy={
            'Name': 'on-failure', 'MaximumRetryCount': 2})

        docker_container = self._get_container_in_docker(container)
        policy = docker_container['HostConfig']['RestartPolicy']
        self.assertEqual('on-failure', policy['Name'])
        self.assertEqual(2, policy['MaximumRetryCount'])

    @decorators.idempotent_id('6b3229af-c8c8-4a11-8b22-981e2ff63b51')
    def test_run_container_with_interactive(self):
        container = self._run_container(interactive=True)

        docker_container = self._get_container_in_docker(container)
        tty = docker_container['Config']['Tty']
        stdin_open = docker_container['Config']['OpenStdin']
        self.assertIs(True, tty)
        self.assertIs(True, stdin_open)

    @decorators.idempotent_id('f189624c-c9b8-4181-9485-2b5cacb633bc')
    def test_reboot_container(self):
        container = self._run_container()
        docker_container = self._get_container_in_docker(container)
        pid = docker_container['State']['Pid']

        self.zun.containers.restart(container.uuid, timeout=10)
        self._ensure_container_pid_changed(container, pid)
        self.assertEqual('Running', self._get_state_in_docker(container))
        # assert pid is changed
        docker_container = self._get_container_in_docker(container)
        self.assertNotEqual(pid, docker_container['State']['Pid'])

    def _ensure_container_pid_changed(self, container, pid):
        def is_pid_changed():
            docker_container = self._get_container_in_docker(container)
            new_pid = docker_container['State']['Pid']
            if pid != new_pid:
                return True
            else:
                return False
        utils.wait_for_condition(is_pid_changed)

    @decorators.idempotent_id('44e28cdc-6b33-4394-b7cb-3b2f36a2839a')
    def test_update_container(self):
        container = self._run_container(cpu=0.1, memory=100)
        self.assertEqual('100', container.memory)
        self.assertEqual(0.1, container.cpu)
        docker_container = self._get_container_in_docker(container)
        self._assert_resource_constraints(docker_container, cpu=0.1,
                                          memory=100)

        container = self.zun.containers.update(container.uuid, cpu=0.2,
                                               memory=200)
        self.assertEqual('200', container.memory)
        self.assertEqual(0.2, container.cpu)
        docker_container = self._get_container_in_docker(container)
        self._assert_resource_constraints(docker_container, cpu=0.2,
                                          memory=200)

    def _assert_resource_constraints(self, docker_container, cpu, memory):
        cpu_shares = docker_container['HostConfig']['CpuShares']
        self.assertEqual(int(cpu * 1024), cpu_shares)
        docker_memory = docker_container['HostConfig']['Memory']
        self.assertEqual(memory * 1024 * 1024, docker_memory)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from mock import patch
from oslo_utils import uuidutils
from webtest.app import AppError

from zun.api.controllers.v1 import capsules
from zun.common import exception
from zun import objects
from zun.tests.unit.api import base as api_base
from zun.tests.unit.db import utils


class TestCheckCapsuleTemplate(api_base.FunctionalTest):
    def test_check_capsule_template(self):
        with self.assertRaisesRegex(
            exception.InvalidCapsuleTemplate, "kind fields need to "
                                              "be set as capsule or Capsule"):
            params = ({"kind": "test", "metadata": {},
                       "spec": {"containers": []}})
            capsules.check_capsule_template(params)

        with self.assertRaisesRegex(
                exception.InvalidCapsuleTemplate, "No Spec found"):
            params = ({"kind": "capsule", "metadata": {}})
            capsules.check_capsule_template(params)

        with self.assertRaisesRegex(
                exception.InvalidCapsuleTemplate,
                "No valid containers field"):
            params = ({"kind": "capsule", "metadata": {}, "spec": {}})
            capsules.check_capsule_template(params)

        params = ({"kind": "capsule", "metadata": {}, "spec": {
            "containers": [{"image": "test1"}], "restartPolicy": "Always",
        }})
        spec_content, tpl_json = capsules.check_capsule_template(params)
        self.assertEqual(spec_content["restart_policy"], "always")

    def test_check_capsule_template_unicode(self):
        with self.assertRaisesRegex(
                exception.SchemaValidationError,
                "Invalid input for field 'kind'"):
            params = (u'{"kind": "test", "metadata": {}, '
                      '"spec": {"containers": []}}')
            capsules.check_capsule_template(params)

        with self.assertRaisesRegex(
                exception.SchemaValidationError,
                "'spec' is a required property"):
            params = (u'{"kind": "capsule", "metadata": {}}')
            capsules.check_capsule_template(params)

        with self.assertRaisesRegex(
                exception.SchemaValidationError,
                "Invalid input for field 'spec'"):
            params = (u'{"kind": "capsule", "spec": {}, "metadata": {}}')
            capsules.check_capsule_template(params)

        params = (u'{"kind": "capsule", "metadata": {}, "spec": {'
                  u'"containers": [{"image": "test1"}],'
                  u'"restartPolicy": "Always"}}')
        spec_content, tpl_json = capsules.check_capsule_template(params)
        self.assertEqual(spec_content["restart_policy"], "always")


class TestCapsuleController(api_base.FunctionalTest):
    @patch('zun.compute.api.API.container_create')
    @patch('zun.network.neutron.NeutronAPI.get_available_network')
    def test_create_capsule(self, mock_capsule_create,
                            mock_neutron_get_network):
        params = ('{'
                  '"template": '
                  '{"kind": "capsule",'
                  ' "spec": {'
                  '  "containers":'
                  '  [{"env": {"ROOT_PASSWORD": "foo0"}, '
                  '    "name": "test-container", '
                  '    "image": "test",'
                  '    "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}'
                  '  }]'
                  ' }, '
                  ' "metadata": {"labels": {"foo0": "bar0", "foo1": "bar1"},'
                  '              "name": "capsule-example"}'
                  ' }'
                  '}')
        response = self.post('/v1/capsules/',
                             params=params,
                             content_type='application/json')
        return_value = response.json
        expected_meta_name = "capsule-example"
        expected_meta_labels = {"foo0": "bar0", "foo1": "bar1"}
        expected_memory = '1024'
        expected_cpu = 1.0
        self.assertEqual(return_value["name"], expected_meta_name)
        self.assertEqual(return_value["labels"], expected_meta_labels)
        self.assertEqual(return_value["memory"], expected_memory)
        self.assertEqual(return_value["cpu"], expected_cpu)
        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_capsule_create.called)
        self.assertTrue(mock_neutron_get_network.called)

    @patch('zun.compute.api.API.container_create')
    @patch('zun.network.neutron.NeutronAPI.get_available_network')
    def test_create_capsule_two_containers(self, mock_capsule_create,
                                           mock_neutron_get_network):
        params = ('{'
                  '"template": '
                  '{"kind": "capsule",'
                  ' "spec": {'
                  '  "containers":'
                  '  [{"image": "test", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}}, '
                  '   {"image": "test1", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}}]'
                  ' }, '
                  ' "metadata": {"labels": {"foo0": "bar0"},'
                  '              "name": "capsule-example"}'
                  ' }'
                  '}')
        response = self.post('/v1/capsules/',
                             params=params,
                             content_type='application/json')
        return_value = response.json
        expected_meta_name = "capsule-example"
        expected_meta_labels = {"foo0": "bar0"}
        expected_memory = '2048'
        expected_cpu = 2.0
        self.assertEqual(return_value["name"],
                         expected_meta_name)
        self.assertEqual(return_value["labels"],
                         expected_meta_labels)
        self.assertEqual(return_value["memory"], expected_memory)
        self.assertEqual(return_value["cpu"], expected_cpu)
        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_capsule_create.called)
        self.assertTrue(mock_neutron_get_network.called)

    @patch('zun.compute.api.API.container_create')
    @patch('zun.api.controllers.v1.capsules.check_capsule_template')
    def test_create_capsule_wrong_kind_set(self, mock_check_template,
                                           mock_capsule_create):
        params = ('{"template": {"kind": "test",'
                  '"spec": {"containers":'
                  '[{"environment": {"ROOT_PASSWORD": "foo0"}, '
                  '"image": "test1", "resources": '
                  '{"requests": {"cpu": 1, "memory": 1024}}}]}, '
                  '"metadata": {"labels": {"foo0": "bar0"}, '
                  '"name": "capsule-example"}}}')
        mock_check_template.side_effect = exception.InvalidCapsuleTemplate(
            "kind fields need to be set as capsule or Capsule")
        response = self.post_json('/capsules/', params, expect_errors=True)
        self.assertEqual(400, response.status_int)
        self.assertFalse(mock_capsule_create.called)

    @patch('zun.compute.api.API.container_create')
    @patch('zun.api.controllers.v1.capsules.check_capsule_template')
    def test_create_capsule_less_than_one_container(self, mock_check_template,
                                                    mock_capsule_create):
        params = ('{"template": {"kind": "capsule",'
                  '"spec": {container:[]}, '
                  '"metadata": {"labels": {"foo0": "bar0"}, '
                  '"name": "capsule-example"}}}')
        mock_check_template.side_effect = exception.InvalidCapsuleTemplate(
            "Capsule need to have one container at least")
        response = self.post_json('/capsules/', params, expect_errors=True)
        self.assertEqual(400, response.status_int)
        self.assertFalse(mock_capsule_create.called)

    @patch('zun.compute.api.API.container_create')
    @patch('zun.api.controllers.v1.capsules.check_capsule_template')
    def test_create_capsule_no_container_field(self, mock_check_template,
                                               mock_capsule_create):
        params = ('{"template": {"kind": "capsule",'
                  '"spec": {}, '
                  '"metadata": {"labels": {"foo0": "bar0"}, '
                  '"name": "capsule-example"}}}')
        mock_check_template.side_effect = exception.InvalidCapsuleTemplate(
            "Capsule need to have one container at least")
        self.assertRaises(AppError, self.post, '/capsules/',
                          params=params, content_type='application/json')
        self.assertFalse(mock_capsule_create.called)

    @patch('zun.compute.api.API.container_create')
    @patch('zun.api.controllers.v1.capsules.check_capsule_template')
    def test_create_capsule_no_container_image(self, mock_check_template,
                                               mock_capsule_create):
        params = ('{"template": {"kind": "capsule",'
                  '"spec": {container:[{"env": '
                  '{"ROOT_PASSWORD": "foo1"}]}, '
                  '"metadata": {"labels": {"foo0": "bar0"}, '
                  '"name": "capsule-example"}}}')
        mock_check_template.side_effect = exception.InvalidCapsuleTemplate(
            "Container image is needed")
        self.assertRaises(AppError, self.post, '/v1/capsules/',
                          params=params, content_type='application/json')
        self.assertFalse(mock_capsule_create.called)

    @patch('zun.compute.api.API.container_create')
    @patch('zun.network.neutron.NeutronAPI.get_available_network')
    def test_create_capsule_with_init_containers(self, mock_capsule_create,
                                                 mock_neutron_get_network):
        params = ('{'
                  '"template": '
                  '{"kind": "capsule",'
                  ' "spec": {'
                  '  "initContainers":'
                  '  [{"image": "test-init", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}}],'
                  '  "containers":'
                  '  [{"image": "test", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}}, '
                  '   {"image": "test1", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}}]'
                  ' }, '
                  ' "metadata": {"labels": {"foo0": "bar0"},'
                  '              "name": "capsule-example"}'
                  ' }'
                  '}')
        response = self.post('/v1/capsules/',
                             params=params,
                             content_type='application/json')
        return_value = response.json
        expected_meta_name = "capsule-example"
        expected_meta_labels = {"foo0": "bar0"}
        expected_memory = '2048'
        expected_cpu = 2.0

        self.assertEqual(return_value["name"],
                         expected_meta_name)
        self.assertEqual(return_value["labels"],
                         expected_meta_labels)
        self.assertEqual(return_value["memory"], expected_memory)
        self.assertEqual(return_value["cpu"], expected_cpu)
        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_capsule_create.called)
        self.assertTrue(mock_neutron_get_network.called)

    @patch('zun.compute.api.API.container_create')
    @patch('zun.network.neutron.NeutronAPI.get_available_network')
    def test_create_capsule_with_two_init_containers(self, mock_capsule_create,
                                                     mock_neutron_get_network):
        params = ('{'
                  '"template": '
                  '{"kind": "capsule",'
                  ' "spec": {'
                  '  "containers":'
                  '  [{"image": "test", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}}],'
                  '  "initContainers":'
                  '  [{"image": "init-test", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}}, '
                  '   {"image": "init-test1", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}}}]'
                  ' }, '
                  ' "metadata": {"labels": {"foo0": "bar0"},'
                  '              "name": "capsule-example"}'
                  ' }'
                  '}')
        response = self.post('/v1/capsules/',
                             params=params,
                             content_type='application/json')
        return_value = response.json
        expected_meta_name = "capsule-example"
        expected_meta_labels = {"foo0": "bar0"}
        expected_memory = '1024'
        expected_cpu = 1.0
        self.assertEqual(return_value["name"],
                         expected_meta_name)
        self.assertEqual(return_value["labels"],
                         expected_meta_labels)
        self.assertEqual(return_value["memory"], expected_memory)
        self.assertEqual(return_value["cpu"], expected_cpu)
        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_capsule_create.called)
        self.assertTrue(mock_neutron_get_network.called)

    @patch('zun.volume.cinder_api.CinderAPI.ensure_volume_usable')
    @patch('zun.volume.cinder_api.CinderAPI.create_volume')
    @patch('zun.compute.api.API.container_create')
    @patch('zun.network.neutron.NeutronAPI.get_available_network')
    def test_create_capsule_with_create_new_volume(self, mock_capsule_create,
                                                   mock_neutron_get_network,
                                                   mock_create_volume,
                                                   mock_ensure_volume_usable):
        fake_volume_id = '3259309d-659c-4e20-b354-ee712e64b3b2'
        fake_volume = mock.Mock(id=fake_volume_id)
        mock_create_volume.return_value = fake_volume
        params = ('{'
                  '"template":'
                  '{"kind": "capsule",'
                  ' "spec":'
                  ' {"containers":'
                  '  [{"env": {"ROOT_PASSWORD": "foo0"}, '
                  '    "image": "test", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}},'
                  '     "volumeMounts": [{"name": "volume1", '
                  '                       "mountPath": "/data1"}]'
                  '   }'
                  '  ],'
                  '  "volumes":'
                  '  [{"name": "volume1",'
                  '    "cinder": {"size": 3, "autoRemove": "True"}'
                  '  }]'
                  ' }, '
                  ' "metadata": {"labels": '
                  '             {"foo0": "bar0", "foo1": "bar1"},'
                  '              "name": "capsule-example"}'
                  ' }'
                  '}')
        response = self.post('/v1/capsules/',
                             params=params,
                             content_type='application/json')
        return_value = response.json
        expected_meta_name = "capsule-example"
        expected_meta_labels = {"foo0": "bar0", "foo1": "bar1"}
        expected_memory = '1024'
        expected_cpu = 1.0
        self.assertEqual(return_value["name"], expected_meta_name)
        self.assertEqual(return_value["labels"], expected_meta_labels)
        self.assertEqual(return_value["memory"], expected_memory)
        self.assertEqual(return_value["cpu"], expected_cpu)
        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_capsule_create.called)
        self.assertTrue(mock_neutron_get_network.called)
        self.assertTrue(mock_create_volume.called)

    @patch('zun.volume.cinder_api.CinderAPI.ensure_volume_usable')
    @patch('zun.volume.cinder_api.CinderAPI.search_volume')
    @patch('zun.compute.api.API.container_create')
    @patch('zun.network.neutron.NeutronAPI.get_available_network')
    def test_create_capsule_with_existed_volume(self, mock_capsule_create,
                                                mock_neutron_get_network,
                                                mock_search_volume,
                                                mock_ensure_volume_usable):
        fake_volume_id = '3259309d-659c-4e20-b354-ee712e64b3b2'
        fake_volume = mock.Mock(id=fake_volume_id)
        mock_search_volume.return_value = fake_volume
        params = ('{'
                  '"template":'
                  '{"kind": "capsule",'
                  ' "spec":'
                  ' {"containers":'
                  '  [{"env": {"ROOT_PASSWORD": "foo0"}, '
                  '    "image": "test", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}},'
                  '     "volumeMounts": [{"name": "volume1", '
                  '                       "mountPath": "/data1"}]'
                  '   }'
                  '  ],'
                  '  "volumes":'
                  '  [{"name": "volume1",'
                  '    "cinder": {"volumeID": '
                  '    "3259309d-659c-4e20-b354-ee712e64b3b2"}'
                  '  }]'
                  ' }, '
                  ' "metadata": {"labels": '
                  '              {"foo0": "bar0", "foo1": "bar1"},'
                  '              "name": "capsule-example"}'
                  ' }'
                  '}')
        response = self.post('/v1/capsules/',
                             params=params,
                             content_type='application/json')
        return_value = response.json
        expected_meta_name = "capsule-example"
        expected_meta_labels = {"foo0": "bar0", "foo1": "bar1"}
        expected_memory = '1024'
        expected_cpu = 1.0
        self.assertEqual(return_value["name"], expected_meta_name)
        self.assertEqual(return_value["labels"], expected_meta_labels)
        self.assertEqual(return_value["memory"], expected_memory)
        self.assertEqual(return_value["cpu"], expected_cpu)
        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_capsule_create.called)
        self.assertTrue(mock_neutron_get_network.called)
        self.assertTrue(mock_ensure_volume_usable.called)
        self.assertTrue(mock_search_volume.called)

    @patch('zun.volume.cinder_api.CinderAPI.create_volume')
    @patch('zun.volume.cinder_api.CinderAPI.ensure_volume_usable')
    @patch('zun.volume.cinder_api.CinderAPI.search_volume')
    @patch('zun.compute.api.API.container_create')
    @patch('zun.network.neutron.NeutronAPI.get_available_network')
    def test_create_capsule_with_two_volumes(self, mock_capsule_create,
                                             mock_neutron_get_network,
                                             mock_search_volume,
                                             mock_ensure_volume_usable,
                                             mock_create_volume):
        fake_volume_id1 = '3259309d-659c-4e20-b354-ee712e64b3b2'
        fake_volume = mock.Mock(id=fake_volume_id1)
        mock_search_volume.return_value = fake_volume
        fake_volume_id2 = 'ef770cfb-349a-483a-97f6-b86e46e344b8'
        fake_volume = mock.Mock(id=fake_volume_id2)
        mock_create_volume.return_value = fake_volume
        params = ('{'
                  '"template":'
                  '{"kind": "capsule",'
                  ' "spec":'
                  ' {"containers":'
                  '  [{"env": {"ROOT_PASSWORD": "foo0"}, '
                  '    "image": "test", "resources": '
                  '    {"requests": {"cpu": 1, "memory": 1024}},'
                  '     "volumeMounts": [{"name": "volume1", '
                  '                       "mountPath": "/data1"},'
                  '                       {"name": "volume2", '
                  '                       "mountPath": "/data2"}]'
                  '   }'
                  '  ],'
                  '  "volumes":'
                  '  [{"name": "volume1",'
                  '    "cinder": {"volumeID": '
                  '    "3259309d-659c-4e20-b354-ee712e64b3b2"}},'
                  '   {"name": "volume2",'
                  '    "cinder": {"size": 3, "autoRemove": "True"}'
                  '  }]'
                  ' }, '
                  ' "metadata": {"labels": {"foo0": "bar0", "foo1": "bar1"},'
                  '              "name": "capsule-example"}'
                  ' }'
                  '}')
        response = self.post('/v1/capsules/',
                             params=params,
                             content_type='application/json')
        return_value = response.json
        expected_meta_name = "capsule-example"
        expected_meta_labels = {"foo0": "bar0", "foo1": "bar1"}
        expected_memory = '1024'
        expected_cpu = 1.0
        self.assertEqual(return_value["name"], expected_meta_name)
        self.assertEqual(return_value["labels"], expected_meta_labels)
        self.assertEqual(return_value["memory"], expected_memory)
        self.assertEqual(return_value["cpu"], expected_cpu)
        self.assertEqual(202, response.status_int)
        self.assertTrue(mock_capsule_create.called)
        self.assertTrue(mock_neutron_get_network.called)
        self.assertTrue(mock_ensure_volume_usable.called)
        self.assertTrue(mock_search_volume.called)
        self.assertTrue(mock_create_volume.called)

    @patch('zun.compute.api.API.container_show')
    @patch('zun.objects.Capsule.get_by_uuid')
    @patch('zun.objects.Container.get_by_uuid')
    def test_get_one_by_uuid(self, mock_container_get_by_uuid,
                             mock_capsule_get_by_uuid,
                             mock_container_show):
        test_container = utils.get_test_container()
        test_container_obj = objects.Container(self.context, **test_container)
        mock_container_get_by_uuid.return_value = test_container_obj
        mock_container_show.return_value = test_container_obj

        test_capsule = utils.create_test_container(context=self.context)
        test_capsule_obj = objects.Capsule(self.context, **test_capsule)
        mock_capsule_get_by_uuid.return_value = test_capsule_obj

        response = self.get('/v1/capsules/%s/' % test_capsule['uuid'])

        context = mock_capsule_get_by_uuid.call_args[0][0]
        self.assertIs(False, context.all_projects)
        self.assertEqual(200, response.status_int)
        self.assertEqual(test_capsule['uuid'],
                         response.json['uuid'])

    @patch('zun.compute.api.API.container_show')
    @patch('zun.objects.Capsule.get_by_uuid')
    @patch('zun.objects.Container.get_by_uuid')
    def test_get_one_by_uuid_all_projects(self, mock_container_get_by_uuid,
                                          mock_capsule_get_by_uuid,
                                          mock_container_show):
        test_container = utils.get_test_container()
        test_container_obj = objects.Container(self.context, **test_container)
        mock_container_get_by_uuid.return_value = test_container_obj
        mock_container_show.return_value = test_container_obj

        test_capsule = utils.create_test_container(context=self.context)
        test_capsule_obj = objects.Capsule(self.context, **test_capsule)
        mock_capsule_get_by_uuid.return_value = test_capsule_obj

        response = self.get('/capsules/%s/?all_projects=1' %
                            test_capsule['uuid'])

        context = mock_capsule_get_by_uuid.call_args[0][0]
        self.assertIs(False, context.all_projects)
        self.assertEqual(200, response.status_int)
        self.assertEqual(test_capsule['uuid'],
                         response.json['uuid'])

    @patch('zun.compute.api.API.container_delete')
    @patch('zun.compute.api.API.container_stop')
    @patch('zun.objects.Capsule.get_by_uuid')
    @patch('zun.objects.Capsule.save')
    def test_delete_capsule_by_uuid(self, mock_capsule_save,
                                    mock_capsule_get_by_uuid,
                                    mock_capsule_stop,
                                    mock_capsule_delete):
        test_capsule = utils.create_test_container(context=self.context)
        test_capsule_obj = objects.Capsule(self.context,
                                           **test_capsule)
        mock_capsule_get_by_uuid.return_value = test_capsule_obj
        mock_capsule_save.return_value = True
        mock_capsule_delete.return_value = True

        capsule_uuid = test_capsule.get('uuid')
        response = self.app.delete('/v1/capsules/%s' % capsule_uuid)

        self.assertTrue(mock_capsule_delete.called)
        self.assertTrue(mock_capsule_stop.called)
        self.assertEqual(204, response.status_int)
        context = mock_capsule_save.call_args[0][0]
        self.assertIs(False, context.all_projects)

    @patch('zun.common.policy.enforce')
    @patch('zun.compute.api.API.container_delete')
    @patch('zun.compute.api.API.container_stop')
    @patch('zun.objects.Capsule.get_by_uuid')
    @patch('zun.objects.Capsule.save')
    def test_delete_capsule_by_uuid_all_projects(self,
                                                 mock_capsule_save,
                                                 mock_capsule_get_by_uuid,
                                                 mock_capsule_stop,
                                                 mock_capsule_delete,
                                                 mock_policy):
        mock_policy.return_value = True
        test_capsule = utils.create_test_container(context=self.context)
        test_capsule_obj = objects.Capsule(self.context,
                                           **test_capsule)
        mock_capsule_get_by_uuid.return_value = test_capsule_obj
        mock_capsule_save.return_value = True
        mock_capsule_delete.return_value = True

        capsule_uuid = test_capsule.get('uuid')
        response = self.app.delete(
            '/v1/capsules/%s/?all_projects=1' % capsule_uuid)

        self.assertTrue(mock_capsule_delete.called)
        self.assertTrue(mock_capsule_stop.called)
        self.assertEqual(204, response.status_int)
        context = mock_capsule_save.call_args[0][0]
        self.assertIs(True, context.all_projects)

    def test_delete_capsule_with_uuid_not_found(self):
        uuid = uuidutils.generate_uuid()
        self.assertRaises(AppError, self.app.delete,
                          '/capsules/%s' % uuid)

    @patch('zun.compute.api.API.container_delete')
    @patch('zun.compute.api.API.container_stop')
    @patch('zun.objects.Capsule.get_by_name')
    @patch('zun.objects.Capsule.save')
    def test_delete_capsule_by_name(self, mock_capsule_save,
                                    mock_capsule_get_by_uuid,
                                    mock_capsule_stop,
                                    mock_capsule_delete):
        test_capsule = utils.create_test_container(context=self.context)
        test_capsule_obj = objects.Capsule(self.context,
                                           **test_capsule)
        mock_capsule_get_by_uuid.return_value = test_capsule_obj
        mock_capsule_save.return_value = True
        mock_capsule_delete.return_value = True

        capsule_name = test_capsule.get('name')
        response = self.app.delete('/v1/capsules/%s/' %
                                   capsule_name)

        self.assertTrue(mock_capsule_delete.called)
        self.assertTrue(mock_capsule_stop.called)
        self.assertEqual(204, response.status_int)
        context = mock_capsule_save.call_args[0][0]
        self.assertIs(False, context.all_projects)

    @patch('zun.compute.api.API.container_show')
    @patch('zun.objects.Capsule.list')
    @patch('zun.objects.Container.get_by_uuid')
    def test_get_all_capsules(self, mock_container_get_by_uuid,
                              mock_capsule_list,
                              mock_container_show):
        test_container = utils.get_test_container()
        test_container_obj = objects.Container(self.context,
                                               **test_container)
        mock_container_get_by_uuid.return_value = test_container_obj

        test_capsule = utils.create_test_container(context=self.context)
        test_capsule_obj = objects.Capsule(self.context, **test_capsule)
        mock_capsule_list.return_value = [test_capsule_obj]
        mock_container_show.return_value = test_container_obj

        response = self.app.get('/v1/capsules/')

        mock_capsule_list.assert_called_once_with(mock.ANY,
                                                  1000, None, 'id', 'asc',
                                                  filters=None)
        context = mock_capsule_list.call_args[0][0]
        self.assertIs(False, context.all_projects)
        self.assertEqual(200, response.status_int)
        actual_capsules = response.json['capsules']
        self.assertEqual(1, len(actual_capsules))
        self.assertEqual(test_capsule['uuid'],
                         actual_capsules[0].get('uuid'))

    @patch('zun.compute.api.API.container_show')
    @patch('zun.objects.Capsule.list')
    @patch('zun.objects.Container.get_by_uuid')
    def test_get_all_capsules_all_projects(self,
                                           mock_container_get_by_uuid,
                                           mock_capsule_list,
                                           mock_container_show):
        test_container = utils.get_test_container()
        test_container_obj = objects.Container(self.context,
                                               **test_container)
        mock_container_get_by_uuid.return_value = test_container_obj

        test_capsule = utils.create_test_container(context=self.context)
        test_capsule_obj = objects.Capsule(self.context, **test_capsule)
        mock_capsule_list.return_value = [test_capsule_obj]
        mock_container_show.return_value = test_container_obj

        response = self.app.get('/v1/capsules/?all_projects=1')

        mock_capsule_list.assert_called_once_with(mock.ANY,
                                                  1000, None, 'id', 'asc',
                                                  filters=None)
        context = mock_capsule_list.call_args[0][0]
        self.assertIs(True, context.all_projects)
        self.assertEqual(200, response.status_int)
        actual_capsules = response.json['capsules']
        self.assertEqual(1, len(actual_capsules))
        self.assertEqual(test_capsule['uuid'],
                         actual_capsules[0].get('uuid'))

    @patch('zun.objects.Capsule.list')
    @patch('zun.objects.Container.get_by_uuid')
    def test_get_all_capsules_with_exception(self,
                                             mock_container_get_by_uuid,
                                             mock_capsule_list):
        test_container = utils.get_test_container()
        test_container_obj = objects.Container(self.context,
                                               **test_container)
        mock_container_get_by_uuid.return_value = test_container_obj

        test_capsule = utils.create_test_container(context=self.context)
        test_capsule_obj = objects.Capsule(self.context, **test_capsule)
        mock_capsule_list.return_value = [test_capsule_obj]

        response = self.app.get('/v1/capsules/')

        mock_capsule_list.assert_called_once_with(mock.ANY,
                                                  1000, None, 'id', 'asc',
                                                  filters=None)
        context = mock_capsule_list.call_args[0][0]
        self.assertIs(False, context.all_projects)
        self.assertEqual(200, response.status_int)
        actual_capsules = response.json['capsules']
        self.assertEqual(1, len(actual_capsules))
        self.assertEqual(test_capsule['uuid'],
                         actual_capsules[0].get('uuid'))

    @patch('zun.compute.api.API.container_show')
    @patch('zun.objects.Capsule.list')
    @patch('zun.objects.Capsule.save')
    @patch('zun.objects.Capsule.get_by_uuid')
    def test_get_all_capsules_with_pagination_marker(
            self,
            mock_container_get_by_uuid,
            mock_capsule_save,
            mock_capsule_list,
            mock_capsule_show):
        test_container = utils.get_test_container()
        test_container_obj = objects.Container(self.context,
                                               **test_container)
        mock_container_get_by_uuid.return_value = test_container_obj
        capsule_list = []
        for id_ in range(4):
            test_capsule = utils.create_test_container(
                id=id_, uuid=uuidutils.generate_uuid(),
                name='capsule' + str(id_), context=self.context)
            capsule_list.append(objects.Capsule(self.context,
                                                **test_capsule))
        mock_capsule_list.return_value = capsule_list[-1:]
        mock_capsule_show.return_value = capsule_list[-1]
        mock_capsule_save.return_value = True

        response = self.app.get('/v1/capsules/?limit=3&marker=%s'
                                % capsule_list[2].uuid)

        self.assertEqual(200, response.status_int)
        actual_capsules = response.json['capsules']

        self.assertEqual(1, len(actual_capsules))
        self.assertEqual(actual_capsules[-1].get('uuid'),
                         actual_capsules[0].get('uuid'))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from oslo_utils import timeutils

from zun.common import context
from zun import objects
from zun.scheduler.filters import compute_filter
from zun.tests import base
from zun.tests.unit.scheduler import fakes


@mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
class TestComputeFilter(base.TestCase):

    def setUp(self):
        super(TestComputeFilter, self).setUp()
        self.context = context.RequestContext('fake_user', 'fake_project')

    def test_compute_filter_manual_disable(self, service_up_mock):
        filt_cls = compute_filter.ComputeFilter()
        container = objects.Container(self.context)
        extra_spec = {}
        service = objects.ZunService(self.context)
        service.disabled = True
        service.disabled_reason = 'This is a reason!'
        host = fakes.FakeHostState('host1',
                                   {'service': service})
        self.assertFalse(filt_cls.host_passes(host, container,
                                              extra_spec))
        self.assertFalse(service_up_mock.called)

    def test_compute_filter_sgapi_passes(self, service_up_mock):
        filt_cls = compute_filter.ComputeFilter()
        container = objects.Container(self.context)
        service = objects.ZunService(self.context)
        service.disabled = False
        extra_spec = {}
        host = fakes.FakeHostState('host2',
                                   {'service': service})
        service_up_mock.return_value = True
        self.assertTrue(filt_cls.host_passes(host, container,
                                             extra_spec))
        service_up_mock.assert_called_once_with(service)

    def test_compute_filter_sgapi_fails(self, service_up_mock):
        filts_cls = compute_filter.ComputeFilter()
        container = objects.Container(self.context)
        service = objects.ZunService(self.context)
        service.disabled = False
        service.updated_at = timeutils.utcnow()
        extra_spec = {}
        host = fakes.FakeHostState('host3',
                                   {'service': service})
        service_up_mock.return_value = False
        self.assertFalse(filts_cls.host_passes(host, container,
                                               extra_spec))
        service_up_mock.assert_called_once_with(service)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools

import copy
from oslo_context import context
from oslo_utils import timeutils
import six

from zun.common import exception
from zun.common import policy


class RequestContext(context.RequestContext):
    """Extends security contexts from the OpenStack common library."""

    def __init__(self, auth_token=None, domain_id=None,
                 domain_name=None, user_name=None, user_id=None,
                 user_domain_name=None, user_domain_id=None,
                 project_name=None, project_id=None, roles=None,
                 is_admin=None, read_only=False, show_deleted=False,
                 request_id=None, trust_id=None, auth_token_info=None,
                 all_projects=False, password=None, timestamp=None, **kwargs):
        """Stores several additional request parameters:

        :param domain_id: The ID of the domain.
        :param domain_name: The name of the domain.
        :param user_domain_id: The ID of the domain to
                               authenticate a user against.
        :param user_domain_name: The name of the domain to
                                 authenticate a user against.

        """
        super(RequestContext, self).__init__(auth_token=auth_token,
                                             user_id=user_name,
                                             project_id=project_name,
                                             is_admin=is_admin,
                                             read_only=read_only,
                                             show_deleted=show_deleted,
                                             request_id=request_id,
                                             roles=roles)

        self.user_name = user_name
        self.user_id = user_id
        self.project_name = project_name
        self.project_id = project_id
        self.domain_id = domain_id
        self.domain_name = domain_name
        self.user_domain_id = user_domain_id
        self.user_domain_name = user_domain_name
        self.auth_token_info = auth_token_info
        self.trust_id = trust_id
        self.all_projects = all_projects
        self.password = password
        if is_admin is None:
            self.is_admin = policy.check_is_admin(self)
        else:
            self.is_admin = is_admin

        if not timestamp:
            timestamp = timeutils.utcnow()
        if isinstance(timestamp, six.string_types):
            timestamp = timeutils.parse_strtime(timestamp)
        self.timestamp = timestamp

    def to_dict(self):
        value = super(RequestContext, self).to_dict()
        value.update({'auth_token': self.auth_token,
                      'domain_id': self.domain_id,
                      'domain_name': self.domain_name,
                      'user_domain_id': self.user_domain_id,
                      'user_domain_name': self.user_domain_name,
                      'user_name': self.user_name,
                      'user_id': self.user_id,
                      'project_name': self.project_name,
                      'project_id': self.project_id,
                      'is_admin': self.is_admin,
                      'read_only': self.read_only,
                      'roles': self.roles,
                      'show_deleted': self.show_deleted,
                      'request_id': self.request_id,
                      'trust_id': self.trust_id,
                      'auth_token_info': self.auth_token_info,
                      'password': self.password,
                      'all_projects': self.all_projects,
                      'timestamp': timeutils.strtime(self.timestamp) if
                      hasattr(self, 'timestamp') else None
                      })
        return value

    def to_policy_values(self):
        policy = super(RequestContext, self).to_policy_values()
        policy['is_admin'] = self.is_admin
        return policy

    @classmethod
    def from_dict(cls, values):
        return cls(**values)

    def elevated(self):
        """Return a version of this context with admin flag set."""
        context = copy.copy(self)
        # context.roles must be deepcopied to leave original roles
        # without changes
        context.roles = copy.deepcopy(self.roles)
        context.is_admin = True

        if 'admin' not in context.roles:
            context.roles.append('admin')

        return context

    def can(self, action, target=None, fatal=True, might_not_exist=False):
        """Verifies that the given action is valid on the target in this context.

        :param action: string representing the action to be checked.
        :param target: dictionary representing the object of the action
            for object creation this should be a dictionary representing the
            location of the object e.g. ``{'project_id': context.project_id}``.
            If None, then this default target will be considered:
            {'project_id': self.project_id, 'user_id': self.user_id}
        :param fatal: if False, will return False when an
            exception.NotAuthorized occurs.
        :param might_not_exist: If True the policy check is skipped (and the
            function returns True) if the specified policy does not exist.
            Defaults to false.

        :raises zun.common.exception.NotAuthorized: if verification fails and
            fatal is True.

        :return: returns a non-False value (not necessarily "True") if
            authorized and False if not authorized and fatal is False.
        """
        if target is None:
            target = {'project_id': self.project_id,
                      'user_id': self.user_id}

        try:
            return policy.authorize(self, action, target,
                                    might_not_exist=might_not_exist)
        except exception.NotAuthorized:
            if fatal:
                raise
            return False


def make_context(*args, **kwargs):
    return RequestContext(*args, **kwargs)


def get_admin_context(show_deleted=False, all_projects=False):
    """Create an administrator context.

    :param show_deleted: if True, will show deleted items when query db
    """
    context = RequestContext(user_id=None,
                             project=None,
                             is_admin=True,
                             show_deleted=show_deleted,
                             all_projects=all_projects)
    return context


def set_context(func):
    @functools.wraps(func)
    def handler(self, ctx):
        if ctx is None:
            ctx = get_admin_context(all_projects=True)
        func(self, ctx)
    return handler



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils

from zun.common import exception
import zun.conf
from zun.db import api as dbapi
from zun.tests.unit.db import base
from zun.tests.unit.db import utils

CONF = zun.conf.CONF


class DbAllocationTestCase(base.DbTestCase):

    def setUp(self):
        super(DbAllocationTestCase, self).setUp()

    def test_create_allocation(self):
        utils.create_test_allocation(context=self.context)

    def test_get_allocation_by_id(self):
        provider = utils.create_test_resource_provider(
            context=self.context)
        allocation = utils.create_test_allocation(
            resource_provider_id=provider.id, context=self.context)
        res = dbapi.get_allocation(self.context, allocation.id)
        self.assertEqual(allocation.id, res.id)

    def test_get_allocation_that_does_not_exist(self):
        allocation_id = 1111111
        self.assertRaises(exception.AllocationNotFound,
                          dbapi.get_allocation,
                          self.context,
                          allocation_id)

    def test_list_allocations(self):
        cids = []
        for i in range(1, 6):
            provider = utils.create_test_resource_provider(
                id=i,
                uuid=uuidutils.generate_uuid(),
                context=self.context)
            allocation = utils.create_test_allocation(
                id=i,
                resource_provider_id=provider.id,
                consumer_id=uuidutils.generate_uuid(),
                context=self.context)
            cids.append(allocation['consumer_id'])
        res = dbapi.list_allocations(self.context)
        res_cids = [r.consumer_id for r in res]
        self.assertEqual(sorted(cids), sorted(res_cids))

    def test_list_allocations_sorted(self):
        cids = []
        for i in range(5):
            provider = utils.create_test_resource_provider(
                id=i,
                uuid=uuidutils.generate_uuid(),
                context=self.context)
            allocation = utils.create_test_allocation(
                id=i,
                resource_provider_id=provider.id,
                consumer_id=uuidutils.generate_uuid(),
                context=self.context)
            cids.append(allocation['consumer_id'])
        res = dbapi.list_allocations(self.context,
                                     sort_key='consumer_id')
        res_cids = [r.consumer_id for r in res]
        self.assertEqual(sorted(cids), res_cids)

        self.assertRaises(exception.InvalidParameterValue,
                          dbapi.list_allocations,
                          self.context,
                          sort_key='foo')

    def test_list_allocations_with_filters(self):
        provider = utils.create_test_resource_provider(
            id=1,
            uuid=uuidutils.generate_uuid(),
            context=self.context)
        allocation1 = utils.create_test_allocation(
            used=0,
            resource_provider_id=provider.id,
            context=self.context)
        allocation2 = utils.create_test_allocation(
            used=1,
            resource_provider_id=provider.id,
            context=self.context)

        res = dbapi.list_allocations(
            self.context, filters={'used': 0})
        self.assertEqual([allocation1.id], [r.id for r in res])

        res = dbapi.list_allocations(
            self.context, filters={'used': 1})
        self.assertEqual([allocation2.id], [r.id for r in res])

        res = dbapi.list_allocations(
            self.context, filters={'used': 11111})
        self.assertEqual([], [r.id for r in res])

        res = dbapi.list_allocations(
            self.context,
            filters={'used': allocation1.used})
        self.assertEqual([allocation1.id], [r.id for r in res])

    def test_destroy_allocation(self):
        allocation = utils.create_test_allocation(context=self.context)
        dbapi.destroy_allocation(self.context, allocation.id)
        self.assertRaises(exception.AllocationNotFound,
                          dbapi.get_allocation,
                          self.context, allocation.id)

    def test_destroy_allocation_by_id(self):
        allocation = utils.create_test_allocation(context=self.context)
        dbapi.destroy_allocation(self.context, allocation.id)
        self.assertRaises(exception.AllocationNotFound,
                          dbapi.get_allocation,
                          self.context, allocation.id)

    def test_destroy_allocation_that_does_not_exist(self):
        allocation_id = 1111111
        self.assertRaises(exception.AllocationNotFound,
                          dbapi.destroy_allocation, self.context,
                          allocation_id)

    def test_update_allocation(self):
        allocation = utils.create_test_allocation(context=self.context)
        old_used = allocation.used
        new_used = 0
        self.assertNotEqual(old_used, new_used)

        res = dbapi.update_allocation(self.context, allocation.id,
                                      {'used': new_used})
        self.assertEqual(new_used, res.used)

    def test_update_allocation_not_found(self):
        allocation_id = 11111111
        new_used = 0
        self.assertRaises(exception.AllocationNotFound,
                          dbapi.update_allocation, self.context,
                          allocation_id, {'used': new_used})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_versionedobjects.tests import test_fields

from zun.objects import fields


class TestContainerStatus(test_fields.TestField):
    def setUp(self):
        super(TestContainerStatus, self).setUp()
        self.field = fields.ContainerStatus()
        self.coerce_good_values = [
            ('Error', 'Error'),
            ('Running', 'Running'),
            ('Stopped', 'Stopped'),
            ('Paused', 'Paused'),
            ('Unknown', 'Unknown'),
            ('Creating', 'Creating'),
            ('Created', 'Created'),
            ('Deleting', 'Deleting'),
        ]
        self.coerce_bad_values = ['bad_value']
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'Error'",
                         self.field.stringify('Error'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'bad_value')


class TestTaskState(test_fields.TestField):
    def setUp(self):
        super(TestTaskState, self).setUp()
        self.field = fields.TaskState()
        self.coerce_good_values = [
            ('image_pulling', 'image_pulling'),
            ('container_creating', 'container_creating'),
        ]
        self.coerce_bad_values = ['bad_value']
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'image_pulling'",
                         self.field.stringify('image_pulling'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'bad_value')


class TestResourceClass(test_fields.TestField):
    def setUp(self):
        super(TestResourceClass, self).setUp()
        self.field = fields.ResourceClass()
        self.coerce_good_values = [
            ('VCPU', 'VCPU'),
            ('MEMORY_MB', 'MEMORY_MB'),
            ('DISK_GB', 'DISK_GB'),
            ('PCI_DEVICE', 'PCI_DEVICE'),
            ('SRIOV_NET_VF', 'SRIOV_NET_VF'),
            ('NUMA_SOCKET', 'NUMA_SOCKET'),
            ('NUMA_CORE', 'NUMA_CORE'),
            ('NUMA_THREAD', 'NUMA_THREAD'),
            ('NUMA_MEMORY_MB', 'NUMA_MEMORY_MB'),
            ('IPV4_ADDRESS', 'IPV4_ADDRESS'),
        ]
        self.coerce_bad_values = ['bad_value']
        self.to_primitive_values = self.coerce_good_values[0:1]
        self.from_primitive_values = self.coerce_good_values[0:1]

    def test_stringify(self):
        self.assertEqual("'VCPU'",
                         self.field.stringify('VCPU'))

    def test_stringify_invalid(self):
        self.assertRaises(ValueError, self.field.stringify, 'bad_value')
